function [Aavg, A] = FeedForwardActivity(stim, neurons, layers, connects, BR, iterations)
%Implements a feed-forward branching network model. 

%Inputs:
%stim - number of neurons to be stimulated in the first layer
%neurons - number of neurons in each layer
%layers - number of layers
%connects - number of connections from each neuron to neurons in the next
%layer
%BR - branching ratio. Should be near 1 to operate near criticality.
%However, for finite sized networks it is necessary to be slightly above 1
%to attain near-critical operation
%iterations - number of networks to be run with these parameters (output is
%the average of all these networks)

%Outputs:
%Aavg - the average activity per layer for the network
%A - the activity per layer for each of the networks that were run

%assign inputs to match code text below
sums = BR; 
c = neurons;
r = layers; 
activity = sparse(r, c);  %binary activity of r*c units, initialized to zeros
 

%set up random connectivity and random synaptic weights
for i=1:r   %for each layer
   for j=1:c  %for each unit in the layer
      holder = [];
      for k=1:connects
         holder = [holder floor(rand*c)+1];
      end
      conmat{i,j} = holder; %conmat{i,j} gives units in next layer getting projections from unit{i,j}
      weightmat{i,j} = PickWeights(connects, sums)'; %weights of connections from unit{i,j} 
   end
end


%now that connectivity and weights are "frozen," propagate activity through the network many times
%sequentially go through each number of neurons to be stimulated
m = stim;
A = sparse(iterations, r);
for l=1:iterations  %do this iterations times for each number of neurons
    activity = sparse(r, c);

    %propagate activity through network

    %propagate input to first layer
    ChosenOnes = randperm(c);
    activity(1,ChosenOnes(1:m)) = 1;   %pick m neurons to be active at start
    stim(m, l) = sum(sum(activity));
    %activity(1,:) = round(rand(1, c));  %random binary input for first layer of network;

    %propagate from first layer to all subsequent layers
    for i=1:r-1 %each layer
        for j=1:c  %each neuron
            for k=1:connects  %each connection
                if (rand <= weightmat{i,j}(k)) & (activity(i,j) ~= 0)
                    activity(i+1,conmat{i,j}(k)) = 1;
                end
            end
        end
        A(l, :) = sum(activity,2)';
    end
end

Aavg = sum(A)/iterations;
s = std(A);



%   figure; plot(Aavg, 'k'); hold on; plot(Aavg + s, 'c'); hold on; plot(Aavg - s, 'c'); hold off
%   title(['Branching ratio = ', num2str(sums)], 'Fontsize', 15); ylabel('Activity per layer', 'FontSize', 15); xlabel('Layer number', 'FontSize', 15);
%   dim = [.5 .5 .3 .3];
%   str = ['Neurons/Layer=', num2str(neurons), ' ', 'Layers=', num2str(layers)];
%   annotation('textbox',dim,'String',str,'FitBoxToText','on');


%   figure; loglog(Aavg, 'k'); hold on; loglog(Aavg + s, 'c'); hold on; loglog(Aavg - s, 'c');hold off
%   title(['Branching ratio = ', num2str(sums)], 'Fontsize', 15); ylabel('Activity per layer', 'FontSize', 15); xlabel('Layer number', 'FontSize', 15);
%   dim = [.15 .15 .3 .3];
%   str = ['Neurons/Layer=', num2str(neurons), ' ', 'Layers=', num2str(layers)];
%   annotation('textbox',dim,'String',str,'FitBoxToText','on');
%   

