function [TIMERASTER] = BranchingModel(timesteps, r, c, p, refperiod, connects, BR, beta)
%Simulates a branching process network
% JMB and Rashid Williams-Garcia 


%timesteps is the number of time 4ms steps, 250 steps per second. 
%1 Million is a reasonable number and takes about 20 s to
%execute with 144 neurons. 

%initial parameters; determines size of "sheet" of neurons
%r = 12;  % rows
%c = 12;  % columns

%p = 0.0005;      %probability of spontaneous activity at a single unit
threshold = 1 - p;  %when membrane potential at a unit is over this, it becomes active for 1 time step
alpha = 1.0; 		%fraction of activity that is transmitted
%BranRatio = BR;   %total sum of synaptic weights (branching ratio)
%refperiod = 10;  %number of time steps that unit is refractory after firing
%connects = 10;  %number of connections that each unit makes to other units. Keep this low to model a few strong connections.
                %but not too low, or there will be no chaining of strong
                %links
%Beta = 0.5;  %This exponent sets the steepness for decay of the connection weights. If
%Beta is zero, all weights are the same; if it is 5, then only the first
%weight is large and all the rest are near zero. 

%set up random connectivity and weight matrices
%cell array connmat has a row for each unit and a column for each output connection
connmat=zeros(connects,r*c);
weightmat=zeros(connects,r*c);

for i=1:r*c
   for j=1:connects
      connmat(j,i) = floor(rand*r*c)+1;
      weightmat(j,i)= exp(-beta*j);
   end
end

weightmat = BR * (weightmat/sum(weightmat(:,1)));

%initialize matricies
spont = zeros(r,c);  %random "membrane potentials" between 0 and 1 for each unit
ref = zeros(r,c);  %lists how many time steps each electrode will be refractory for
act = zeros(r,c);  %gives driven activity at each unit, set by spont and connectivity matrix
current = zeros(r,c); %status of activity at current time step 
next = zeros(r,c);  %status of activity at next time step
TIMERASTER = sparse(r*c, timesteps);  %stores times of events for plotting later using LFPsorter routines

%march through time steps
for i=1:timesteps
    
   %determine spontaneous activity over threshold
   spont(rand(r,c) >= threshold)=1;
   
   %determine units to be activated next time step by current activtiy that is over threshold
   for j=1:r*c
       if current(j) ~= 0
         for k=1:connects
            if rand <= weightmat(k,j)
               act(connmat(k,j)) = act(connmat(k,j)) + (alpha * current(j));  %update act matrix
            end
         end
       end
   end

   act(act >= threshold) = 1;   %trim act to keep it from accumulating large values
      
   %determine activity at next time step
   next = spont + act;
   next(ref > 0) = 0;  %turn off all units that are already refractory
   next(current >= threshold) = 0;        %turn off all units in next time step that fired in current time step
   next(next >= threshold) = 1;        %make next have binary values only
   next(next < threshold) = 0;
   
   %determine refractory units for next time step
   ref = ref-1;  %decrement old values
   ref(ref < 0) = 0;
   ref(next >= threshold) = refperiod;  %increment new values
      
   %update current matrix and clear other matricies
   current = next;
   next = zeros(r,c);
   act = zeros(r,c);
   spont = zeros(r,c);
   TIMERASTER(:,i) = reshape(current,r*c,1);
end
