function [Shapes] = ShapeCollapse(E, LimL, LimU, interval, gamma_act)
%To extract average avalanche shapes from a data set that has already been
%passed through AvalancheAnalysis. The output of that function that serves
%as input to this one includes the avalanche size distribution (S), the
%avalanche duration distribution (D), the average avalanche size for a
%given duration vs the duration (SvT) and the data containing information
%about all the avalanche events (E). Also needed for input are the lower
%limit (LimL) and upper limit (LimU) of the avalanche durations to be
%considered for plotting. To avoid crowding the plot, you can plot
%avalanche durations separated by interval. For example, you could skip
%every other avalanche if interval = 2. To plot all, interval = 1.
%gamma_act is the actual gamma exponent measured by the function
%ExponentRelation. 
%As output, this function plots the average avalanche shapes within the 
%specified domain (Shapes), and also plots the collapsed shapes. If they
%approximately follow the same shapes, regardless of scale, then they can be
%considered fractal copies of each other, further evidence of scale-free
%properties produced by the system. 

%Get avalanche statistics from Events
Events = E;
for i=1:length(Events)
        Durations(i) = length(Events{i});
        Areas(i) = sum(Events{i});
end

%Get and plot average avalanche size against avalanche duration
%Get average avalanche shapes
%First collect all shapes
Shapes = cell([max(Durations), 1]);
for i=1:length(Events)
    if Durations(i) ~= 0
        Shapes{Durations(i)} = [Shapes{Durations(i)}; Events{i}];
    end
end
%Second, average them
for i=1:max(Durations)
    if ~isempty(Shapes{i})
        [rw cl] = size(Shapes{i});
        holder = sum(Shapes{i})/rw;
        Shapes{i} = holder;
    end
end



%plot average shapes without rescaling
figure;
for i=LimL:interval:LimU     %:1:21 %MaxDuration
    plot(Shapes{i}); hold on
end
ylabel('Avalanche amplitude', 'FontSize', 15); xlabel('Avalanche duration', 'FontSize', 15);
title('Average avalanche shapes', 'Fontsize', 15); 
axis('square')



%plot collapsed shapes
figure; 
for i=LimL:interval:LimU  %MaxDuration
    if ~isempty(Shapes{i})
        holder = Shapes{i}/(length(Shapes{i})^(gamma_act-1));
        times = [1:i]/i;
        
        %adjust times so that middle value is centered at 0.5
        middleIndex = floor(length(times)/2) + 1;
        adjust = times(middleIndex) - 0.5;
        
        plot(times - adjust, holder); hold on
    end
end
xlim([-0.1 1.1]); 
axis('square')
ylabel('Scaled avalanche amplitude', 'FontSize', 15); xlabel('Scaled avalanche duration (t/T)', 'FontSize', 15);
title('Collapsed avalanche shapes', 'Fontsize', 15); 

