function [info, Hresp, Hcond] = FindInformation(stimulus, response, c)
%Finds average mutual information between an ensemble of input stimuli and the corresponding 
%ensemble of reponses. c is the number of units (binary digits) in the input layer of the network.
%Stimulus and response are binary input and output configurations that have been converted 
%to decimal integers. Info calculated using I[S; R] = H(R) - H(R|S) (See e.g., Zador, 1998),
%where H(R) is the variability of the responses over the ensemble of inputs and H(R|S)
%is the reliability of the response to a particular input.  High information demands
%high response variety to a variety of inputs, yet repeatable responses when the same 
%input stimuli are given. Output given in bits.


%Error check; find length of input
if length(stimulus) ~= length(response)
   error('Stimulus and response must be the same length.');
else
   lin = length(stimulus);
end


%I. Find H(R)
%find response probabilities
pout = sparse(1, (2^c));
for i=1:(2^c)
   [Y, ~] = find(response == i-1);
   pout(i) = length(Y);   %note that pout(i) contains info about the (i-1)th probability
end                     %this is because inputs run from say 0-15, and indicies must be > 0.
pout = pout/sum(pout);

%calculate the entropy of the responses
Hresp = 0;
for i=1:(2^c)
   holder = 0;
   if pout(i) == 0
      holder = 0;
   else
      holder = -1 * (pout(i)) * ((log10(pout(i)))/(log10(2)));
   end
   Hresp = Hresp + holder;
end


%II. Find H(R|S)
%find conditional probabilities
pcond = sparse((2^c), (2^c));
for i=1:(2^c)
   [Y, I] = find(stimulus == i-1);
   if length(I) > 0
      for j=1:length(I)
         pcond(response(I(j))+1, i) = pcond(response(I(j))+1, i) + 1;
      end
   end
   if sum(pcond(:,i)) ~= 0
       pcond(:,i) = pcond(:,i)/sum(pcond(:,i));
   end
end

%calculate entropy of the conditional responses
Hcond = sparse((2^c), 1);
for i=1:(2^c)
   holder = 0;
   for j=1:(2^c)
      if pcond(j,i) == 0
         holder = holder + 0;
      else
         holder = holder + (-1 * (pcond(j,i)) * ((log10(pcond(j,i)))/(log10(2)))); 
      end
   end
   Hcond(i) = holder;
end
Hcond = sum(Hcond)/(2^c);


%III. Find info 
info = Hresp - Hcond;

