function [stim, resp] = RecurrentInfoTester(r, c, connects, sums, reps, timesteps, N)
%Propagates activity in a recurrent network for timesteps times. Will output a list of the
%stimulus configurations used as well as the response configurations for
%each layer. 
%Uses RecurrentModel, bin2dec2

%Explain parameters
%r = rows
%c = columns the total number of neurons in the "sheet" is r x c
%connects = number of connections from each neuron
%sums = sum of transmission probabilities from each neuron. At criticality
%when sums = 1 in an infinite network. 
% reps = number of iterations the network is run
% timesteps = number of timesteps in each run
% N = number of neurons to be used as input and output

%Here, reps and timesteps are added to the input specifications
%and are not embedded in the program. reps gives the number of times the 
%entire stimulus set is presented to the given net, while timesteps gives the 
%number of timesteps the network is allowed to evolve before the output vector is taken.
%N is the number of units that are to be randomly chosen as input and
%output, without overlap. This allows a huge network to be studied by only
%sampling a subset of its units.
   
%Set initializations
activity = zeros(r, c);  %binary activity of r*c units, initialized to zeros
avg = zeros(1,c);  %holds averages
devs = zeros(1,c);  %holds standard deviations

%set up random connectivity and weight matrices
%cell array connmat has a row for each unit and a column for each output
%connection
   for i=1:r*c %go through all the units in serial order. Ex: if it is an 8 x 8 matrix, go through units 1:64
      holder = [];
      for j=1:connects
         holder = [holder floor(rand*r*c) + 1]; 
      end
      connmat{i} = holder;                      %connmat{i} holds indices of units that unit i will project to
      weightmat{i} = weights2(connects, sums)'; %weightmat{i} holds weights of connections from unit{i}
   end
   clear holder

   
%randomly select N input units (inunits) and N output units (outunits) for the
%network. These will be a subset of the entire network, allowing a huge
%network to be probed by testing only a subset of its elements at a time.
%Randomly choose N input units and N non-overlapping output units from r*c
%units available
unitList = 1:(r*c);
InputUnits = [];
for i=1:N
    [Y I] = max(rand(1, length(unitList)));
    InputUnits = [InputUnits unitList(I)];
    holder = remove(unitList', unitList(I));
    unitList = holder';
end

OutputUnits = [];
for i=1:N
    [Y I] = max(rand(1, length(unitList)));
    OutputUnits = [OutputUnits unitList(I)];
    holder = remove(unitList', unitList(I));
    unitList = holder';
end
clear holder

%select inputconfig. This is the activity configuration of all the
%"background" units of the network that were not chosen as InputUnits or
%OutputUnits. This inputconfig should be random,
%50% ones and 50% zeros, and kept the same each time the network is run.
%This way, the program is examining how a random subset of neurons can
%transmit information to another random subset of neurons in a background
%of noise. 
inputconfig = sparse(r,c);
inputconfig(find(rand(r,c) > 0.5)) = 1;

%cycle through all of the input states
for s=0:(2^(N))-1  %if there are r*c units in the network, there are 2^(r*c) possible states
   %convert decimal integer s into binary input vector
   holder = dec2bin(s);       
   adjust = (N) - length(holder);
   for ind=1:adjust
      holder = ['0' holder];
   end
   for ind=1:(N)
      input(ind) = str2num(holder(((N)+1)-ind));
   end
         
   %now that connectivity and weights are "frozen," propagate activity through the network many times
   for m=1:reps     %to get a rough sampling, propagate each state through network reps times
     TIMERASTER = RecurrentModel(input, inputconfig, connects, connmat, weightmat, timesteps, InputUnits, OutputUnits);
     for j=1:timesteps
         resp(j,(s*reps) + m) = bin2dec2(TIMERASTER(:,j));
         stim(j,(s*reps) + m) = s;
     end
   end
end

