function IsingTriangle(T, r, timesteps)
%Periodic boundary conditions 
%Modified from IsingSquare, which had a square lattice. Now a
%triangular lattice is used. This is accomplished by adding two diagonal
%connections from the central site. This is topologically equivalent to a
%triangular lattice. Each central site now has six nearest neighbors,
%instead of four, as was the case with the square lattice. 
%Should pick at least 1000 timesteps or there will be no movie


%I. Preliminaries: create matrix and randomly populate it with spins up or down
c = r; %number of columns
interval = 300; %how many timesteps before you take a snapshot for the movie

%II. Set up matrices. In a ferromagnet, assume nearest neighbor
%interactions only, and assume that all interactions are positive (this
%will tend to cause spins to align in the same direction). Assume periodic
%boundary conditions.
%first, create mapping square
x = 1:(r*c);
Sq = reshape(x, r, c);
Sq2 = [zeros(r,1) Sq zeros(r,1)];
Sq2 = [zeros(1,c+2); Sq2; zeros(1,c+2)];
Sq2(:,1) = Sq2(:,c+1); Sq2(:,c+2) = Sq2(:,2); %columns wrap around
Sq2(1,:) = Sq2(r+1,:); Sq2(r+2,:) = Sq2(2,:); %rows wrap around


%Now go through the inside of this square and create a nearest neighbor
%connectivity matrix based on these toroidal neighbors
%Conn is connectivity matrix: Conn(i,j) gives the connection between Mat(i)
%and Mat(j) when elements are indexed in serial order
Conn = sparse(r*c, r*c);
for i=2:r+1
    for j=2:c+1
        Conn(Sq2(i,j), Sq2(i-1,j)) = 1; %North
        Conn(Sq2(i,j), Sq2(i+1,j)) = 1; %South
        Conn(Sq2(i,j), Sq2(i,j-1)) = 1; %West
        Conn(Sq2(i,j), Sq2(i,j+1)) = 1; %East
        Conn(Sq2(i,j), Sq2(i-1,j-1)) = 1; %NorthWest
        Conn(Sq2(i,j), Sq2(i+1,j+1)) = 1; %SouthEast
    end
end
  
for i=1:r*c
    J = find(Conn(i,:) ~= 0);
    ConnList{i} = J;
end


%initialize the spin matrix Mat1 with random +1, -1 values
Mat1 = rand(r, c);
Mat1(find(Mat1 > 0.5)) = 1;
Mat1(find(Mat1 <= 0.5)) = -1;
Mat2 = Mat1;  %initialize Mat2, the matrix for the next time step

%Capture spin values of initial matrix
counter = 1;
SnapShots{counter} = Mat2; 
counter = counter + 1; 

%Initialize some arrays for keeping records
DensityOfSites = zeros(timesteps, 1);
Elog = zeros(timesteps, 1); 
dElog = zeros(timesteps, 1); 
CorrMat = zeros(r, r, floor(timesteps/interval)+1);


%III. Main loop. Go through the matrix, randomly picking a site, and determine whether to flip a spin
%or not. Use Metropolis algorithm. Briefly, retain a spin flip if it lowers
%energy, unless the temperature is high. 
for time=1:timesteps %note this is simulation time, not actual time
    time
    E = 0;
    %Calculate total energy
    for i=1:(r*c)   %go through all spins
        for j=1:6   %easch spin has *six* neighbors
             E = E - (Mat1(i)*Mat1(ConnList{i}(j))); % assume connection strengths are 1 (J = 1). 
        end
    end
    
    %Randomly pick a spin to flip
    Chosen = randperm(r*c); %Chosen(1) will be the chosen one
    
    %Evaluate energy change caused by spin flip  
    dE = 0;
    for j=1:6   %each spin has *six* neighbors, energy change will be -2*the energy associated with that spin
        dE = dE - (-2*Mat1(Chosen(1))*Mat1(ConnList{Chosen(1)}(j))); %the -1 is the spin flip
    end
    
    %Decide whether to retain spin flip or not
    if dE <= 0 %always retain flip if it lowers energy
        Mat2(Chosen(1)) = -1*Mat1(Chosen(1));  %the -1 is the spin flip
        E = E + dE;
    else
        if rand < exp(-1*(dE/T)) %if flip does not lower energy, still retain it if meets Boltzmann criterion
            Mat2(Chosen(1)) = -1*Mat1(Chosen(1));
            E = E + dE;
        end
    end
    
    Mat1 = Mat2;  %update matrix
    
    %store state of magnet for later graphical display
    if (mod(time, interval)== 0) %& (timesteps >= 1000) %allow at least 1000 timesteps to pass to allow transients to die out
        SnapShots{counter} = Mat2;  %load each frame for later movie
        CorrMat(:,:, counter) = Mat2;
        counter = counter + 1;
    end

    %store other information 
    Elog(time) = E/2; %divide by two to prevent double counting the connections' contributions to total energy
    dElog(time) = dE;
    DensityOfSites(time) = sum(sum((Mat1 + ones(r, r))/2))/(r * r);
end

%Calculate susceptibility, Chi, based on density of states:
Chi = var(DensityOfSites); 

%IV. Play movie
figure; 
startind = 1; %this is the index of the first frame you will view
frames = length(SnapShots); %number of time steps you want to look at
axis square;
M = moviein(frames);
set(gca,'NextPlot','replacechildren')

 for i=1:frames  
   frame = SnapShots{i}; 
   greysB = [((1/64)*(64:-1:1))' ((1/64)*(64:-1:1))' ((1/64)*(64:-1:1))']; %black on top
   chosenmap = greysB;colormap(chosenmap); 
   image(100*frame); axis square; set(gca,'Visible', 'off');
   M(:,i) = getframe;
 end


%V. Get correlation function against distance, specific heat, susceptibility

%Get average correlation between sites as a function of distance
AvgR = [];
for i=1:r
    N = reshape(CorrMat(i,:,:), r, []);
    R = cov(N'); 
    posList = [i:r 1:i-1];
    AvgR = [AvgR; R(i,posList)];
end
AR = sum(AvgR)/r;
CenteredAR = [AR(floor(r/2)+1:r) AR(1:floor(r/2))];
if mod(r, 2) == 0
    figure; plot([-(floor(r/2)):1:(floor(r/2)-1)], CenteredAR); ylabel('Covariance', 'FontSize', 15); xlabel('Distance, lattice sites', 'FontSize', 15); ylim([-0.5 1]); title('T =', T);
else
    figure; plot([-(floor(r/2)+1):1:(floor(r/2)-1)], CenteredAR); ylabel('Covariance', 'FontSize', 15); xlabel('Distance, lattice sites', 'FontSize', 15); ylim([-0.5 1]); title('T =', T);
end

%Calculate the specific heat
SpHeat = var(Elog(floor(timesteps/5):timesteps))/(T^2); %only measure this after transient

%Calculate susceptibility
Chi = var(DensityOfSites); 

%VI. Save file 
fname = ['IsingTriangle_T' num2str(100*T) 'r' num2str(r) 'timesteps' num2str(timesteps)];
save(fname,'M', 'r', 'T', 'timesteps', 'CorrMat', 'DensityOfSites', 'Chi', 'Elog', 'dElog', 'CorrMat', 'CenteredAR', 'SpHeat')

