function [TIMERASTER] = BP_Function(Layers, timesteps) 
%Implements a Branching Process (SOBP) described in
%Zapperi et al., 1995, Physical Review Letters, but here *without* self-organization. 
%A binary tree will be set up: one node in the first layer, two in the
%second, four in the third, etc. The first layer node is excited and then
%activity propagates through the tree until it stops. There is one
%transmission probability for all connections at a given time t, P. 
%This is fixed at P = 0.5. If each node has only two outgoing
%connections, this sets a branching ratio of BR = 2 * P(t) = 1.0

%Layers is the number of layers in the binary tree
%timesteps is the number of iterations the model is run (the total number
%of avalanches)


%Reasonable starting parameters: Layers = 12, timesteps = 50000, A = 1;
%This took about 35 seconds on my laptop to run. Making Layers and
%timesteps large will of course take more time, but it scales in a
%nonlinear way, so it gets long very quickly!


tic; %Start timer to time how long it takes
%Set up size of tree
L = Layers;       %Number of layers in the tree
N = zeros(1, (2^L) - 1); %Nodes of the tree. Node is 1 if on, 0 if off. If there are 4 layers, then 15 nodes
                    %Use linear numbering for nodes, with first node being
                    %1, two nodes in next layer being 2 and 3; four nodes in
                    %next layer being 4 through 7, etc. Note that node N is
                    %connected to nodes 2N and 2N+1 in the next layer with
                    %this scheme.

iterations = timesteps; %number of iterations
TIMERASTER = sparse(1, (L+1)*iterations); 

%Select initial tranmission probability and vector for future transmission
%probabilities
P = 0.5;
S = zeros(1, iterations); %array to hold avalanche sizes
T = zeros(1, iterations); %array to hold avalanche durations

%create list of nodes by layer
for i=1:L
    NodesByLayer{i} = [2^(i-1):((2^i)-1)]; %for second layer, nodes 2, 3. For third layer, nodes 4, 5, 6, 7, etc.
end

counter = 1;
for t=1:iterations
    %Always activate first node
    N(1) = 1;
    ActiveInLayer = 1; %keeps track of number of active nodes per layer. Start out with activity in first layer

    %Now propagate through the rest of the tree by going through the layers.
    flag = 1;
    i=1;
    while (flag == 1) & (i <= L) %i is the current layer

        %find all active nodes in current layer
        nodes = NodesByLayer{i}; 
        [Y I] = find(N(nodes) == 1); %range(I) are the active nodes

        if isempty(I) | i == L %If no active nodes, avalanche is stopped OR avalanche has reached end of network
            flag = 0;
            S(t) = sum(N); %get size of avalanche
            T(t) = i;      %get duration
            %get events for later calculation of shape
            len = length(ActiveInLayer);
            TIMERASTER(1, counter:(counter + len - 1)) = ActiveInLayer;
            counter = counter + len + 1;
   
        else %If at least one node is active
            %get list of nodes in the next layer that could possibly receive transmission
            nodes = NodesByLayer{i+1};

            %transmit probabilistically to next layer from active nodes in current layer
            [Z J] = find(rand(1, 2*(length(I))) < P); % J is the list of connections that transmit
            N(nodes(J)) = 1; %activate those that transmitted
            if sum(N(nodes(J))) ~= 0 %append to ActiveInLayer only if it is nonzero
                ActiveInLayer = [ActiveInLayer sum(N(nodes(J)))];
            end
            i = i + 1; %now may move on to the next layer
        end
    end

    %reset N
    N = zeros(1, (2^L) - 1); %reset N

end

toc  %end timer