function [alpha, tau, gamma_est, gamma_act, error] = ExponentRelation(CCS, CCD, SvT, LimL, LimU)
%ExponentRelation takes CCDF of size distribution (CCS) and duration
%distribution (CCD) produced by GetCCDFs. It also takes the size vs. 
%duration (SvT) data structures produced by AvalancheAnalysis. 
%Lastly, it must be given a lower limit (LimL) and an
%upper limit (LimU) that have been determined by viewing the CCDF plots. 
%It returns the avalanche duration distribution exponent (alpha) and the
%size distribution exponent (tau) estimated from the CCDFs over the range
%specified by the lower and uppper limits (LimL, LimU). From alpha and tau,
%it calculates an estimated exponent (gamma_est) based on the exponent 
%relation (1-alpha)/(1-tau) = gamma_est. It also estimates the exponent 
%gamma from the size vs duration data (SvT) as gamma_act. By comparing 
%gamma_est withgamma_act, it calculates the relative error in satisfying 
%the exponent relation: error = abs((gamma_act - gamma_est)/gamma_act).

%Transfer variables
lowerLim = LimL;
upperLim = LimU; 

%% Fit the duration distribution CCDF over the domain selected
Y = CCD;
y = 1:length(Y);
ex = log10(y(lowerLim:upperLim));
why = log10(Y(lowerLim:upperLim));
figure; plot(ex,why,'LineWidth',2); ylabel('Log(CCDF of Avalanche sizes)', 'FontSize', 15); xlabel('Log(Avalanche size)', 'FontSize', 15);
title('CCDF of Avalanche sizes', 'Fontsize', 15);

[c f] = polyfit(ex, why,1);
% Display evaluated equation y = m*x + b
disp(['Equation is y = ' num2str(c(1)) '*x + ' num2str(c(2))])
disp(['Norm of the residuals is = ' num2str(f.normr)])
alpha = c(1) - 1; %Exponent of the avalanche size distribution. Because cumsum is the integral, must adjust exponent by adding  -1

% Evaluate fit equation using polyval
y_est = polyval(c,ex);
% Add trend line to plot
hold on
plot(ex,y_est,'r--','LineWidth',2); title('Line fit', 'Fontsize', 15);
hold off

%% Fit the size distribution CCDF over the domain selected
clear y Y
Y = CCS;
y = 1:length(Y);
ex = log10(y(lowerLim:upperLim));
why = log10(Y(lowerLim:upperLim));
figure; plot(ex,why,'LineWidth',2); ylabel('Log(CCDF of Avalanche durations)', 'FontSize', 15); xlabel('Log(Avalanche duration)', 'FontSize', 15);
title('CCDF of Avalanche durations', 'Fontsize', 15);

[c f] = polyfit(ex, why,1);
% Display evaluated equation y = m*x + b
disp(['Equation is y = ' num2str(c(1)) '*x + ' num2str(c(2))])
disp(['Norm of the residuals is = ' num2str(f.normr)])
tau = c(1) - 1; %Exponent of avalanche duration distribution. Because cumsum is the integral, must adjust exponent by adding  -1

% Evaluate fit equation using polyval
y_est = polyval(c,ex);
% Add trend line to plot
hold on
plot(ex,y_est,'r--','LineWidth',2); ylabel('Log(CCDF of Avalanche durations)', 'FontSize', 15); xlabel('Log(Avalanche durations)', 'FontSize', 15);
title('Line fit', 'Fontsize', 15);
hold off

%% Fit the size for a given duration vs the duration (SvsT) over the domain selected
ex = log10(SvT(1, lowerLim:upperLim));
why = log10(SvT(2, lowerLim:upperLim));
figure; plot(ex,why,'LineWidth',2)

[c f] = polyfit(ex, why,1);
% Display evaluated equation y = m*x + b
disp(['Equation is y = ' num2str(c(1)) '*x + ' num2str(c(2))])
disp(['Norm of the residuals is = ' num2str(f.normr)])
gamma_act = c(1); %Exponent of avg. avalanche size vs. duration plot

% Evaluate fit equation using polyval
y_est = polyval(c,ex);
% Add trend line to plot
hold on
plot(ex,y_est,'r--','LineWidth',2); ylabel('Log(Avg. Avalanche size)', 'FontSize', 15); xlabel('Log(Avalanche duration)', 'FontSize', 15);
title('Line fit', 'Fontsize', 15);
hold off

%% Estimate error in adherence to the exponent relation
gamma_est = (abs(alpha) -1)/(abs(tau) -1);
error = abs((gamma_act - gamma_est)/gamma_act);

