function [TIMERASTER, BranchingRatio] = SOBPfunction(Layers, timesteps, A, Pstart) 
%Implements the Self-Organized Branching Process (SOBP) described in
%Zapperi et al., 1995, Physical Review Letters. 
%A binary tree will be set up: one node in the first layer, two in the
%second, four in the third, etc. The first layer node is excited and then
%activity propagates through the tree until it stops. There is one
%transmission probability for all connections at a given time t, P(t). This starts out at
%some random value, P(1), but through homeostatic feedback it is brought close to the
%critical value of P(iterations) = 0.5. If each node has only two outgoing
%connections, this should converge to a branching ratio of BR = 2 * P(t) = 1.0
%after many iterations.

%Layers is the number of layers in the binary tree
%timesteps is the number of iterations the model is run (the total number
%of avalanches)
%A is the learning rate, usually set near one or slightly less. It controls
%how rapidly the transmission probabilities are adjusted. 
%Pstart is the starting probability, which could be from 0 to 1. The system
%will converge on a probability of P = 0.5 when critical. 

%Reasonable starting parameters: Layers = 12, timesteps = 50000, A = 1;
%Pstart = 0.65;


tic; 
%Set up size of tree
L = Layers;       %Number of layers in the tree
N = zeros(1, (2^L) - 1); %Nodes of the tree. Node is 1 if on, 0 if off. If there are 4 layers, then 15 nodes
                    %Use linear numbering for nodes, with first node being
                    %1, two nodes in next layer being 2 and 3; four nodes in
                    %next layer being 4 through 7, etc. Note that node N is
                    %connected to nodes 2N and 2N+1 in the next layer with
                    %this scheme.

iterations = timesteps; %number of iterations
TIMERASTER = sparse(1, (L+1)*iterations); 
TF = 0.2; %TF for transient fraction - fraction of iterations that may be consumed with coming to equilibrium

%Select initial tranmission probability and vector for future transmission
%probabilities
P = zeros(1, iterations);
P(1) = Pstart; %This is the starting probability which can range anywhere from 0 to 1. The system will converge on P = 0.5.
S = zeros(1, iterations); %array to hold avalanche sizes
T = zeros(1, iterations); %array to hold avalanche durations

%create list of nodes by layer
for i=1:L
    NodesByLayer{i} = [2^(i-1):((2^i)-1)]; %for second layer, nodes 2, 3. For third layer, nodes 4, 5, 6, 7, etc.
end

counter = 1;
counter2 = 2;
for t=1:iterations
    %Always activate first node
    N(1) = 1;
    ActiveInLayer = 1; %keeps track of number of active nodes per layer. Start out with activity in first layer

    %Now propagate through the rest of the tree by going through the layers.
    flag = 1;
    i=1;
    while (flag == 1) & (i <= L) %i is the current layer

        %find all active nodes in current layer
        nodes = NodesByLayer{i}; 
        [Y I] = find(N(nodes) == 1); %range(I) are the active nodes

        if isempty(I) | i == L %If no active nodes, avalanche is stopped OR avalanche has reached end of network
            flag = 0;
            S(t) = sum(N); %get size of avalanche
            T(t) = i;      %get duration
            %get events for later calculation of shape
            len = length(ActiveInLayer);
            TIMERASTER(1, counter:(counter + len - 1)) = ActiveInLayer;
            counter = counter + len + 1;
   
        else %If at least one node is active
            %get list of nodes in the next layer that could possibly receive transmission
            nodes = NodesByLayer{i+1};

            %transmit probabilistically to next layer from active nodes in current layer
            [Z J] = find(rand(1, 2*(length(I))) < P(t)); % J is the list of connections that transmit
            N(nodes(J)) = 1; %activate those that transmitted
            if sum(N(nodes(J))) ~= 0 %append to ActiveInLayer only if it is nonzero
                ActiveInLayer = [ActiveInLayer sum(N(nodes(J)))];
            end
            i = i + 1; %now may move on to the next layer
        end
    end

    %Adjust the transmission probabilities homeostatically, and allow
    %perturbations at selected times

    P(t+1) = P(t) + A*(1-sum(N(2^(L-1):2^L - 1)))/(length(N));
    N = zeros(1, (2^L) - 1); %reset N

end

toc

%Plot evolution of transmission probability. Critical value in
%thermodynamic limit will be P = 0.5
figure; plot(P); ylim([0 1]);
xlabel('time steps'); ylabel('P');

BranchingRatio = 2*mean(P(floor(TF*iterations):iterations));




