function [TIMERASTER] = BranchingProcessFunction(Layers, BR, iterations, Pspont)
%Simulates a branching process on a binary tree with Layers. Branching
%ratio is given by BR, number of runs is given by iterations. Pspont gives
%the probability of a neuron in the tree spontaneously firing. 

%function [TIMERASTER] = BPfunction2(Layers, iterations, Pspont) 
%Implements a Branching Process (SOBP) described in
%Zapperi et al., 1995, Physical Review Letters, but here *without* self-organization. 
%A binary tree will be set up: one node in the first layer, two in the
%second, four in the third, etc. The first layer node is excited and then
%activity propagates through the tree until it stops. There is one
%transmission probability for all connections at a given time t, P. 
%This is fixed at P = 0.5. If each node has only two outgoing
%connections, this sets a branching ratio of BR = 2 * P(t) = 1.0
%Pspont is the probability of each neuron spontaneously firing during an
%avalanche. Spontaneous here means "without being driven." This can be set
%to zero or to values ranging from 1e-5 to 1e-2, for example. 

%Layers is the number of layers in the binary tree
%timesteps is the number of iterations the model is run (the total number
%of avalanches)


%Reasonable starting parameters: Layers = 12, timesteps = 50000, A = 1;
%This took about 35 seconds on my laptop to run. Making Layers and
%timesteps large will of course take more time, but it scales in a
%nonlinear way, so it gets long very quickly!


%Get transmission probability. This will just be half the branching ratio
%in this binary tree
P = BR/2; 

%Set up size of tree
L = Layers;       %Number of layers in the tree
N = zeros(1, (2^L) - 1); %Active nodes of the tree in the current avalanche.
                    %To be filled in as the avalanche propagates through
                    %layers. Node is 1 if on, 0 if off. If there are 4 layers, then 15 nodes
                    %Use linear numbering for nodes, with first node being
                    %1, two nodes in next layer being 2 and 3; four nodes in
                    %next layer being 4 through 7, etc. Note that node N is
                    %connected to nodes 2N and 2N+1 in the next layer with
                    %this scheme.


TIMERASTER = sparse((2^L)-1, (L+1)*iterations); %one row for each neuron, one column for each layer (plus a buffer of one between avalanches) for each timestep


%Select initial tranmission probability and vector for future transmission
%probabilities

S = zeros(1, iterations); %array to hold avalanche sizes
T = zeros(1, iterations); %array to hold avalanche durations

%create list of nodes by layer; in layer i, NodesByLayer{i} lists only
%those nodes in that layer
for i=1:L
    NodesByLayer{i} = [2^(i-1):((2^i)-1)]; %for second layer, nodes 2, 3. For third layer, nodes 4, 5, 6, 7, etc.
end

%counter = 1;
for t=1:iterations
   
    if mod(t, 500) == 0
        waitbar(t/iterations)  %display progress toward task
    end


    %Always activate first node
    N(1) = 1;
    TIMERASTER(1, (1+(t-1)*(L+1))) = 1; %place the activity of the first node in the TIMERASTER

    %Now propagate through the rest of the tree by going through the layers.
    flag = 1;  %When flag == 1, avalanche is still running; when flag == 0, avalanche has stopped
    i=1; %i is the layer that is presently being considered for that iteration t. Ex: t = 567, but i = 3 means we are on the 3rd layer of the network in the 567th iteration
    
    while (flag == 1) & (i <= L) %i is the current layer

        %Deal with active nodes in current layer
        %NodesByLayer{i} are the nodes in the ith layer (which could possibly transmit to the next layer)
        
        %Allow spontaneous activity in current layer
        [X, K] = find(rand(length(NodesByLayer{i}), 1) < Pspont); %X gives list of spontaneously active node indices within NodesByLayer 
        if ~isempty(X) %if some indices were chosen, then activate the corresponding spontaneous nodes in that layer 
            N(NodesByLayer{i}(X)) = 1;
        end
        %Identify all active nodes in current layer
        [Y I] = find(N(NodesByLayer{i}) == 1); %NodesByLayer{i}(I) are the nodes in the ith layer that are active now from either spontaneous or previous driven activity
        TIMERASTER(NodesByLayer{i}(I), (1+(t-1)*(L+1))+(i-1)) = 1; %indicate active nodes in TIMERASTER by placing 1s there


        %Consider what to do next: either terminate the avalanche OR
        %activate the next layer

        %Terminate
        if isempty(I) | i == L %If no active nodes in ith layer OR avalanche has reached end of network, avalanche is stopped
            flag = 0;
            S(t) = sum(N); %get size of avalanche
            T(t) = i;      %get duration
   
        %Activate next layer    
        else %If at least one node is active (I is not empty) AND i is not yet equal to L
            
            %NodesByLayer{i=1} is the list of nodes in the next layer that could possibly receive transmission
            [Z J] = find(rand(1, 2*(length(I))) < P); % J is the list of connections that transmit
            if ~isempty(J) %if some indices were chosen, then activate the corresponding spontaneous nodes in that layer
                N(NodesByLayer{i+1}(J)) = 1;
            end

            TIMERASTER(NodesByLayer{i+1}(J), (1+(t-1)*(L+1))+(i)) = 1; %place those active nodes in the TIMERASTER
            i = i + 1; %now may move on to the next layer
            
        end
    end

    %reset N
    N = zeros(1, (2^L) - 1); %reset N

end

