function [TIMERASTER, BranchingRatio] = SOBP_Pspont_function(Layers, timesteps, A, Pspont)
%SOBP_Pspont_script
%now trying to add Pspont to nodes beyond the first initial node

%Implements the Self-Organized Branching Process (SOBP) described in
%Zapperi et al., 1995, Physical Review Letters. 
%A binary tree will be set up: one node in the first layer, two in the
%second, four in the third, etc. The first layer node is excited and then
%activity propagates through the tree until it stops. There is one
%transmission probability for all connections at a given time t, P(t). This starts out at
%some random value, P(1), but through homeostatic feedback it is brought close to the
%critical value of P(iterations) = 0.5. If each node has only two outgoing
%connections, this should converge to a branching ratio of BR = 2 * P(t) = 1.0
%after many iterations.

%Layers is the number of layers in the binary tree
%timesteps is the number of iterations the model is run (the total number
%of avalanches)
%A is the learning rate, usually set near one or slightly less. It controls
%how rapidly the transmission probabilities are adjusted. 
%Pspont is the probability of a neuron spontaneously firing

%Reasonable starting parameters: Layers = 14, timesteps = 100000, A = 1;
%Pspont = 0.05;


tic; 
%Set up size of tree, Pspont
%Pspont = 0.15; % Probability of a node in a layer after the first becoming spontaneously active. 
L = Layers;       %Number of layers in the tree
N = zeros(1, (2^L) - 1); %Nodes of the tree. Node is 1 if on, 0 if off. If there are 4 layers, then 15 nodes
                    %Use linear numbering for nodes, with first node being
                    %1, two nodes in next layer being 2 and 3; four nodes in
                    %next layer being 4 through 7, etc. Note that node N is
                    %connected to nodes 2N and 2N+1 in the next layer with
                    %this scheme.

iterations = timesteps; %number of iterations
TF = 0.2; %TF for transient fraction - fraction of iterations that may be consumed with coming to equilibrium
%A = 0.1; %learning rate (For stability, may want this to be less than 1 when using L < 18) 

%Select initial tranmission probability and vector for future transmission
%probabilities
P = zeros(1, iterations);
P(1) = rand; %This can also be P(1) = 0.9; to approach from above, or P(1) = rand; to start randomly

S = zeros(1, iterations); %array to hold avalanche sizes
T = zeros(1, iterations); %array to hold avalanche durations

%create list of nodes by layer
for i=1:L
    NodesByLayer{i} = [2^(i-1):((2^i)-1)]; %for second layer, nodes 2, 3. For third layer, nodes 4, 5, 6, 7, etc.
end


for t=1:iterations
    %Always activate first node
    N(1) = 1;
    ActiveInLayer = 1; %keeps track of number of active nodes per layer. Start out with activity in first layer

    %Now propagate through the rest of the tree by going through the layers.
    flag = 1;
    i=1;
    while (flag == 1) & (i <= L) %i is the current layer

        %find all active nodes in current layer
        nodes = NodesByLayer{i}; 
        [X, J] = find(rand(length(NodesByLayer{i}), 1) <= Pspont); %X gives list of spontaneously active node indices within NodesByLayer 
        if ~isempty(X) %if some indices were chosen, then activate the corresponding nodes in that layer 
            N(X) = 1;
        end
        [Y I] = find(N(nodes) == 1); %range(I) are the active nodes

        if isempty(I) | i == L %If no active nodes, avalanche is stopped OR avalanche has reached end of network
            flag = 0;
            S(t) = sum(N); %get size of avalanche
            T(t) = i;      %get duration
            Events{t} = ActiveInLayer; %get events for later calculation of shape
   
        else %If at least one node is active
            %get list of nodes in the next layer that could possibly receive transmission
            nodes = NodesByLayer{i+1};

            %transmit probabilistically to next layer from active nodes in current layer
            [Z J] = find(rand(1, 2*(length(I))) < P(t)); % J is the list of connections that transmit
            N(nodes(J)) = 1; %activate those that transmitted
            if sum(N(nodes(J))) ~= 0 %append to ActiveInLayer only if it is nonzero
                ActiveInLayer = [ActiveInLayer sum(N(nodes(J)))];
            end
            i = i + 1; %now may move on to the next layer
        end
    end

    %Adjust the transmission probabilities homeostatically
    P(t+1) = P(t) + A*(1-sum(N(2^(L-1):2^L - 1)))/(length(N));
    N = zeros(1, (2^L) - 1); %reset N

end



%Produce a TIMERASTER out of all the Events recorded
TIMERASTER = sparse(1, (L+1)*iterations); 
counter = 1;
for i=1:iterations

    if mod(i, 1000) == 0
        waitbar(i/iterations)  %display progress toward task
    end

    len = length(Events{i});
    TIMERASTER(1, counter:(counter + len - 1)) = Events{i};
    counter = counter + len + 1;
end


%Plot evolution of transmission probability. Critical value in
%thermodynamic limit will be P = 0.5
figure; plot(P); ylim([0 1]);
xlabel('time steps'); ylabel('P');


BranchingRatio = 2*mean(P(floor(TF*iterations):iterations))  %output the branching ratio

toc

figure; plot(S)  %plot avalanche sizes over time
xlabel('time steps'); ylabel('Avalanche sizes');

